#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <popt.h>
#include <string.h>
#include <locale.h>

#include "jisasc.h"

#define BUFMAX 1024

#define APP_NAME "jisasc"
#define JISASC_VERSION "0.1"

typedef enum {
  DUMP_MODE_NOT = -1,
  DUMP_MODE_HEADER,
  DUMP_MODE_2WAY_TEXT,
  DUMP_MODE_ZEN2HAN_TEXT,
  DUMP_MODE_HAN2ZEN_TEXT,
} DumpMode;

void
show_version() {
  g_print("%s - Version %s\n", g_get_prgname(), JISASC_VERSION);
  g_print("Copyright (C) 2003 Yukihiro Nakai <nakai@gnome.gr.jp>.\n");
  exit(0);
}

void
usage(poptContext optCon, int exitcode, char* error) {
  poptPrintUsage(optCon, stderr, 0);
  if( error )
    fprintf(stderr, "ERROR: %s\n", error);
  exit(exitcode);
}

int main(int argc, char** argv) {
  char c = 0;
  int i = 0;
  char buf[BUFSIZ+1];
  char* filename = NULL;
  char* zenfile = NULL;
  char* hanfile = NULL;
  char buffer[BUFMAX];
  poptContext optCon;
  FILE* fp;
  DumpMode dump = DUMP_MODE_NOT;
  gboolean jisasc_show_version = FALSE;

  struct poptOption optionsTable[] = {
    { "zen2han", 'z', POPT_ARG_STRING, &zenfile, 0, "Zenkaku to Hankaku" },
    { "han2zen", 'h', POPT_ARG_STRING, &hanfile, 0, "Hankaku to Zenkaku" },
    { "version", 'v', POPT_ARG_VAL, &jisasc_show_version, TRUE, "Show Version" },
    { "dump-header", '\0', POPT_ARG_VAL, &dump, DUMP_MODE_HEADER, "Dump conversion table as an include header" },
    { "dump-2way-text", '\0', POPT_ARG_VAL, &dump, DUMP_MODE_2WAY_TEXT, "Dump 2way table in text format" },
    { "dump-zen2han-text", '\0', POPT_ARG_VAL, &dump, DUMP_MODE_ZEN2HAN_TEXT, "Dump Zenkaku to Hankaku table in text format" },
    { "dump-han2zen-text", '\0', POPT_ARG_VAL, &dump, DUMP_MODE_HAN2ZEN_TEXT, "Dump Hankaku to Zenkaku table in text format" },
      POPT_AUTOHELP { NULL, '\0', POPT_ARG_NONE, NULL, 0 },
  };

  g_set_prgname(APP_NAME);

  setlocale(LC_ALL, "");

  convtable_init();

  optCon = poptGetContext(NULL, argc, (const char**)argv, optionsTable, 0);

  if( argc < 2 ) {
    poptPrintUsage(optCon, stderr, 0);
    exit(1);
  }

  while((c = poptGetNextOpt(optCon)) >= 0) {
    switch(c) {
    case 'z':
      buf[i++] = 'z';
      break;
    case 'h':
      buf[i++] = 'h';
      break;
    case 'v':
      show_version();
      exit(0);
      break;
    }
  }

  if( jisasc_show_version )
	show_version();

  switch(dump) {
  case DUMP_MODE_HEADER:
    dump_header();
    exit(0);
    break;
  case DUMP_MODE_2WAY_TEXT:
    dump_2way_text();
    exit(0);
    break;
  case DUMP_MODE_ZEN2HAN_TEXT:
    exit(0);
    break;
  case DUMP_MODE_HAN2ZEN_TEXT:
    exit(0);
    break;
  default:
    /* Do nothing */
    break;
  }

  if( !(poptPeekArg(optCon) == NULL) ) {
    usage(optCon, 1, "Specify a single filename with option.");
  }

  if( c < -1 ) {
    fprintf(stderr, "%s: %s\n",
            poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
            poptStrerror(c));
    return 1;
  }

  (zenfile != NULL) ?  (filename = zenfile) : (filename = hanfile);

  fp = fopen(filename, "r");
  if( fp == NULL ) {
    fprintf(stderr, "File error: %s\n", filename);
    exit(1);
  }

  while(fgets(buffer, BUFMAX, fp)) {
    if( !g_utf8_validate(buffer, -1, NULL) ) {
      printf("%s", buffer); /* No conversion */
      continue;
    };
    if( zenfile != NULL ) { /* Zenkaku to Hankaku */
      gchar* hankaku = zen2han(buffer);
      printf("%s", hankaku);
      g_free(hankaku);
    } else { /* Hankaku to Zenkaku */
      gchar* zenkaku = han2zen(buffer);
      printf("%s", zenkaku);
      g_free(zenkaku);
    }
    memset(buffer, 0, BUFMAX);
  }

  fclose(fp);

  poptFreeContext(optCon);

  return 0;
}
